@@ -8,11 +8,10 @@
 #include "pinyinencoder.h"
 #include "shuangpindata.h"
 #include <boost/algorithm/string.hpp>
-#include <exception>
 #include <fcitx-utils/charutils.h>
-#include <iostream>
 #include <set>
 #include <string_view>
+#include <unordered_map>
 
 namespace libime {
 
@@ -104,13 +103,9 @@ ShuangpinProfile::ShuangpinProfile(std::istream &in)
         if (boost::starts_with(lineView, option)) {
             std::string name{lineView.substr(option.size())};
             boost::trim_if(name, isSpaceCheck);
-            if (name == "自然码" || name == "微软" || name == "紫光" ||
-                name == "拼音加加" || name == "中文之星" || name == "智能ABC" ||
-                name == "小鹤") {
-                isDefault = true;
-            } else {
-                isDefault = false;
-            }
+            isDefault = (name == "自然码" || name == "微软" || name == "紫光" ||
+                         name == "拼音加加" || name == "中文之星" ||
+                         name == "智能ABC" || name == "小鹤");
         }
 
         if (isDefault) {
@@ -175,8 +170,12 @@ void ShuangpinProfile::buildShuangpinTable() {
     for (const auto &p : d->initialMap_) {
         d->validInputs_.insert(p.first);
     }
+    std::unordered_map<PinyinFinal, char> singleCharFinal;
     for (const auto &p : d->finalMap_) {
         d->validInputs_.insert(p.first);
+        if (PinyinEncoder::finalToString(p.second).size() == 1) {
+            singleCharFinal[p.second] = p.first;
+        }
     }
 
     for (const auto &p : d->initialFinalMap_) {
@@ -213,10 +212,11 @@ void ShuangpinProfile::buildShuangpinTable() {
     std::set<char> finalChars;
     for (auto c = PinyinEncoder::firstFinal; c <= PinyinEncoder::lastFinal;
          c++) {
-        const auto &finalString =
-            PinyinEncoder::finalToString(static_cast<PinyinFinal>(c));
-        if (finalString.size() == 1) {
+        auto f = static_cast<PinyinFinal>(c);
+        const auto &finalString = PinyinEncoder::finalToString(f);
+        if (finalString.size() == 1 && !singleCharFinal.count(f)) {
             finalChars.insert(finalString[0]);
+            singleCharFinal[f] = finalString[0];
         }
     }
     // Add final in map to finalChars
@@ -224,6 +224,15 @@ void ShuangpinProfile::buildShuangpinTable() {
         finalChars.insert(p.first);
     }
 
+    for (const auto &[final, chr] : singleCharFinal) {
+        auto [begin, end] = d->finalMap_.equal_range(chr);
+        if (std::find_if(begin, end, [final = final](const auto &item) {
+                return item.second == final;
+            }) == end) {
+            d->finalMap_.emplace(chr, final);
+        }
+    }
+
     auto addPinyinToList =
         [](std::multimap<PinyinSyllable, PinyinFuzzyFlags> &pys,
            PinyinInitial i, PinyinFinal f, PinyinFuzzyFlags flags) {
@@ -285,16 +294,6 @@ void ShuangpinProfile::buildShuangpinTable() {
         // length 2: keep same as quanpin
         // length 3: use the initial of quanpin and the one in the table.
         for (auto c : finalChars) {
-            // Check if final char is already a final (e.g. aeiou).
-            auto final = PinyinEncoder::stringToFinal(std::string{c});
-            if (final != PinyinFinal::Invalid &&
-                PinyinEncoder::isValidInitialFinal(PinyinInitial::Zero,
-                                                   final)) {
-                std::string input{c, c};
-                d->spTable_[input].emplace(
-                    PinyinSyllable{PinyinInitial::Zero, final},
-                    PinyinFuzzyFlag::None);
-            }
             // If c is in final map.
             auto finalIterPair = d->finalMap_.equal_range(c);
             for (auto &item : boost::make_iterator_range(
@@ -302,12 +301,21 @@ void ShuangpinProfile::buildShuangpinTable() {
                 if (PinyinEncoder::isValidInitialFinal(PinyinInitial::Zero,
                                                        item.second)) {
                     std::string input;
-                    const auto finalString =
+                    const auto &finalString =
                         PinyinEncoder::finalToString(item.second);
                     if (finalString.size() == 1) {
                         input = std::string{c, c};
                     } else {
-                        input = std::string{finalString[0], c};
+                        auto final = PinyinEncoder::stringToFinal(
+                            std::string{finalString[0]});
+                        if (final != PinyinFinal::Invalid) {
+                            auto singleCharFinalIter =
+                                singleCharFinal.find(final);
+                            if (singleCharFinalIter != singleCharFinal.end()) {
+                                input =
+                                    std::string{singleCharFinalIter->second, c};
+                            }
+                        }
                     }
                     d->spTable_[input].emplace(
                         PinyinSyllable{PinyinInitial::Zero, item.second},
@@ -342,18 +350,9 @@ void ShuangpinProfile::buildShuangpinTable() {
             }
 
             auto finalIterPair = d->finalMap_.equal_range(c2);
-            if (finalIterPair.first != finalIterPair.second) {
-                for (auto &item : boost::make_iterator_range(
-                         finalIterPair.first, finalIterPair.second)) {
-                    finals.push_back(item.second);
-                }
-            }
-            auto final = PinyinEncoder::stringToFinal(std::string{c2});
-            if (!d->finalSet_.count(final)) {
-                // v is defined in map.
-                if (final != PinyinFinal::Invalid) {
-                    finals.push_back(final);
-                }
+            for (auto &item : boost::make_iterator_range(
+                     finalIterPair.first, finalIterPair.second)) {
+                finals.push_back(item.second);
             }
 
             for (auto i : initials) {
@@ -380,6 +379,11 @@ void ShuangpinProfile::buildShuangpinTable() {
 
     // Add non-existent 2 char pinyin to the map.
     for (const auto &p : getPinyinMapV2()) {
+        // Don't add "ng" as two char direct pinyin.
+        if (p.pinyin() == "ng") {
+            continue;
+        }
+
         if (p.pinyin().size() == 2 && p.initial() == PinyinInitial::Zero &&
             (!d->spTable_.count(p.pinyin()) ||
              d->zeroS_.find('*') != std::string::npos)) {
@@ -403,24 +407,27 @@ void ShuangpinProfile::buildShuangpinTable() {
             addPinyinToList(pys, item.second, PinyinFinal::Invalid,
                             PinyinFuzzyFlag::None);
         }
-        auto final = PinyinEncoder::stringToFinal(std::string{c});
-        if (final != PinyinFinal::Invalid &&
-            PinyinEncoder::isValidInitialFinal(PinyinInitial::Zero, final) &&
-            pys.empty()) {
-            addPinyinToList(pys, PinyinInitial::Zero, final,
-                            PinyinFuzzyFlag::None);
+
+        // Add single char final to partial pinyin.
+        auto [begin, end] = d->finalMap_.equal_range(c);
+        for (auto &item : boost::make_iterator_range(begin, end)) {
+            const auto final = item.second;
+            if (PinyinEncoder::finalToString(final).size() == 1 &&
+                PinyinEncoder::isValidInitialFinal(PinyinInitial::Zero,
+                                                   final) &&
+                pys.empty()) {
+                addPinyinToList(pys, PinyinInitial::Zero, final,
+                                PinyinFuzzyFlag::None);
+            }
         }
 
-        // do not look into finalMap_
-        // because we're doing sp, using sp with single final shouldn't be
-        // allowed
         if (pys.empty()) {
             d->spTable_.erase(input);
         }
     }
 
     for (const auto &sp : d->spTable_) {
-        assert(sp.first.size() >= 1 && sp.first.size() <= 2);
+        assert(!sp.first.empty() && sp.first.size() <= 2);
         d->validInitials_.insert(sp.first[0]);
     }
 }
